enum {
	/* one-time stacks */
	Readstack = 8192+MAXMSG+MAXFDATA,	// buffer on stack
	Stitchstack = 8192,
	Rdwrstack = 8192+MAXMSG+MAXFDATA,	// buffer on stack

	Maxfid = 0xff00,
	Maxtag = 0xff00,

	Reqmagic = 0xfeedface,
	Fidmagic = 0xfeedbabe,
};

typedef struct Intpool Intpool;
typedef struct Map Map;
typedef struct Maplist Maplist;
typedef struct Readarg Readarg;
typedef struct Req Req;
typedef struct Session Session;
typedef struct Stitcharg Stitcharg;

struct Map {
	ulong	magic;
	QLock;
	Maplist	*list;
};

/* an extant 9P connection */
struct Session {
	char	*name;
	int	fd;
	int	fsno;
	int	mcount;		/* internal use */
	Channel	*reqchan;	/* chan(Req*) */
	Channel	*rdchan;	/* chan(Req*), internal use */
	/*
	 * we need to use more fids and tags to talk to our servers
	 * (e.g., reading union directories) than our clients use talking
	 * to us, so we need our own.
	 */
	Intpool	*fidpool;
	Intpool	*tagpool;	/* internal use */
	Map	*tagmap;	/* internal use */

	uchar	serverchal[CHALLEN];
	uchar	clientchal[CHALLEN];
	char	authid[NAMELEN];
	char	authdom[DOMLEN];
	char	tbuf[2*TICKETLEN+AUTHENTLEN];
};

/*
 * an extant fid on a 9P connection (to a server).
 * only copy Fiddata, not Fidmeta, when copying Fids.
 * we allocate these, so they shouldn't be shared and
 * there shouldn't be races accessing them.  the allocator
 * locks the Fid pool internally.
 */
typedef struct {
#ifdef PARANOID
	QLock	flock;
#endif
	Ref	ref;		/* actually, it looks like fids aren't shared */
} Fidmeta;
typedef struct {
	Session	*s;
	int	fid;		/* in our fid space for session s */
	Qid	qid;		/* mostly for the CHDIR bit */
	ulong	magic;
} Fiddata;
typedef struct {
	Fidmeta;
	Fiddata;
} Fid;

/*
 * a request or reply.
 * only one thread works on a given Req at a time, except that flushes
 * operate upon active Reqs in other threads; we use the lock in the
 * Map that returned the Req to protect it.
 */
struct Req {
	Ref;
	ulong	magic;
	Fcall;
	Channel	*c;
	char	*buf;	/* MAXMSG+MAXFDATA, usually */
	int	flushed;
};

/* argument to read9pproc */
struct Readarg {
	int	fd;
	Channel	*c;	/* chan(Req*) */
	char	*name;
};

struct Stitcharg {
	int	srvfd;
	int	otherpipeend;
	int	numfs;	/* # of file servers (other than this program) */
	struct Stitchfs {
		int	fsfd;
		char	*fsname;
	} fs[Maxfs];
};

enum {
	/* per-request stacks */
	Reqstack = 8192+sizeof(Req),	// BUG: should be smaller
};

extern char *eve;
extern int chatty, singleth;
