#include <u.h>
#include <libc.h>
#include <auth.h>
#include <fcall.h>
#include <thread.h>
#include "layer.h"
#include "dat.h"
#include "fns.h"

int chatty, singleth;
char *eve;

void *
emallocz(ulong size, int clr)
{
	void *p = mallocz(size, clr);

	if (p == nil)
		sysfatal("malloc(%lud): %r", size);
	return p;
}

void *
emalloc(ulong n)
{
	return emallocz(n, 1);
}

char *
estrdup(char *s)
{
	char *ns = strdup(s);

	if (ns == nil)
		sysfatal("malloc(%lud): out of memory", strlen(s)+1);
	return ns;
}

void
nnfree(void **pp)
{
	if (*pp != nil) {
		free(*pp);
		*pp = nil;
	}
}

void
usage(void)
{
	threadprint(2,
		"usage: %s [-abcmpsv] /srv/top ... /srv/bottom /mnt/point\n",
		argv0);
	exits("usage");
}

static int
opensrv(char *srvfile)
{
	int fd = open(srvfile, ORDWR);

	if (fd < 0)
		sysfatal("open %s: %r", srvfile);
	return fd;
}

void
post(char *srv, int fd)
{
	int f;

	threadprint(2, "post...");
	f = create(srv, OWRITE, 0666);
	if (f < 0) {
		threadprint(2, "%s: create(%s): %r\n", argv0, srv);
		threadexitsall("can't post");
	}
	if (threadprint(f, "%d", fd) < 0) {
		threadprint(2, "%s: write(%s): %r\n", argv0, srv);
		threadexitsall("post write error");
	}
	close(f);
}

void
threadmain(int argc, char **argv)
{
	int i, flags = 0, launchpid, p[2], dopost = 1, domount = 1;
	char srvbuf[NAMELEN+10], *srvname = srvbuf;
	Stitcharg *arg;

	eve = getuser();
	snprint(srvbuf, sizeof srvbuf, "/srv/ovfs.%s.%d", eve, getpid());

	ARGBEGIN {
	case 'a':
		flags |= MAFTER;
		break;
	case 'b':
		flags |= MBEFORE;
		break;
	case 'c':
		flags |= MCREATE;
		break;
	case 'C':
		flags |= MCACHE;
		break;
	case 'm':
		domount = 0;
		break;
	case 'n':
		srvname = ARGF();
		break;
	case 'p':
		dopost = 0;
		break;
	case 's':
		singleth = 1;
		break;
	case 'v':
		chatty = 1;
		break;
	default:
		usage();
	} ARGEND
	if(!(flags & (MAFTER|MBEFORE)))
		flags |= MREPL;

	fmtinstall('F', fcallconv);
	fmtinstall('D', dirconv);

	if (argc < 3)
		usage();

	rfork(RFREND);
	if (pipe(p) < 0)
		sysfatal("pipe: %r");
	if (dopost)
		post(srvname, p[1]);

	/*
	 * start the 9P request launcher process,
	 * with a pipe from the served namespace
	 */
	arg = emalloc(sizeof *arg);
	arg->numfs = argc - 1;
	for (i = 0; i < argc - 1; i++) {
		char *argnm = argv[i];
		struct Stitchfs *fs = &arg->fs[i];

		fs->fsname = estrdup(argnm);
		fs->fsfd = opensrv(argnm);
	}
	arg->srvfd = p[0];
	arg->otherpipeend = p[1];
	launchpid = proccreate(stitchthread, arg, Stitchstack);	// process #4
	if (launchpid < 0)
		sysfatal("proccreate: %r");
	yield();
	threadprint(2, "launcher pid: %d\n", launchpid);

	if (domount) {
		threadprint(2, "mount...");
		if (mount(p[1], argv[argc-1], flags, "") < 0)
			sysfatal("mount pipe on %s: %r", argv[argc-1]);
	} else
		sleep(1000);	// hope stitchread forks in this time

	threadprint(2, "\n");
	threadexits(nil);	// leave other threads running in background
}
