.TH OVFS 4
.SH NAME
ovfs, stitch \- serve overlaid view of file servers
.SH SYNOPSIS
.B ovfs
[
.B \-abcCmpsv
] [
.B \-n
service
]
.BI /srv/ first
.BI /srv/ second
[ ... ]
.I mountpoint
.br
.B stitch
.I dir1
.I dir2
[ ... ]
.I mountpoint
.SH DESCRIPTION
.I Ovfs
serves a view of the file server
.I first
bound over the file server
.I second
and so on,
mounted at
.IR mountpoint .
A pipe to the resulting name space is posted in
.BR /srv .
The result resembles the effect of binding the common directories
except that binding is deferred until files are used,
the binding is done internally to
.I ovfs
instead of in the kernel,
and files are always created in
.IR first .
.SS "Ovfs options"
.PD 0
.TP 3
.B a
mount after.
.TP
.B b
mount before.
.TP
.B c
allow creation in the merged namespace in
.IR mountpoint .
.TP
.B C
cache file server data.
Be careful to only cache access to permanent storage,
not served files nor devices.
.TP
.B m
suppress mounting on
.IR mountpoint .
.TP
.B n
post pipe at
.BI /srv/ service.
.TP
.B p
suppress posting in
.BR /srv .
.TP
.B s
single-thread: don't spawn a thread per client 9P request.
.TP
.B v
be verbose.
.PD
.PP
.I Stitch
is similar but runs
.IR srvfs (4)
on each directory argument
that isn't actually a posted file server connection per
.IR srv (3),
then invokes
.I ovfs
on the resulting
.CW /srv
files and
.IR mountpoint .
.SH EXAMPLES
Apply your private changes and build a private version of a program
without altering the master tree:
.EX
.in +0.5i
stitch src/cmd/scuzz /sys/src/cmd/scuzz tmp/priv
cd tmp/priv
mk
.in -0.5i
.EE
.SH FILES
.PD 0
.TP
.B /srv
contains posted file server connections
.TP
.BI /srv/ovfs. user . pid
connection to resulting served (merged) namespace
.PD
.SH SEE ALSO
.IR ovmk (1),
.IR exportfs (4)
.SH HISTORY
Russ Cox wrote a partial first draft;
Geoff Collyer finished and debugged it.
.SH BUGS
To assist programs that need to be able to figure
out the layering, the uppermost bits of each served qid path
(other than CHDIR) are the ordinal number
of the underlying file server, so the qid path space of
those file servers is limited.
.PP
To conserve qid path bits, the maximum number of file servers
that may be overlaid is limited.
.PP
The
.B /srv
files supplied as arguments must be for as-yet-unmounted
(actually
.RI un-attach (5) ed)
file server connections.
The kernel enforces this restriction in the mount driver.
