#include <u.h>
#include <libc.h>
#include <auth.h>
#include <fcall.h>
#include <thread.h>
#include "layer.h"
#include "dat.h"
#include "fns.h"

// processes #1-3: reads 9P from a file server (v->fd), forwards Reqs over v->c
void
read9pproc(void *v)
{
	int n;
	char *buf = nil;
	Readarg *a = v;
	Req *r = nil;

	for (; ; ) {
		if (buf == nil)
			buf = emalloc(MAXMSG+MAXFDATA);
		if (r == nil)
			r = newreq(nil, 0);
		if ((n = read9p(a->fd, buf, MAXMSG+MAXFDATA)) <= 0) {
			sendp(a->c, nil);
			break;
		}
		if (convM2S(buf, &r->Fcall, n) != n) {
			threadprint(2, "warning: bad message\n");
			continue;
		}
		if (chatty)
			threadprint(2, "read  %s %F\n", a->name, &r->Fcall);
		if (r->type == Rread || r->type == Twrite) {
			r->buf = buf;
			buf = nil;
		} else
			r->buf = nil;

		sendp(a->c, r);
		r = nil;
	}
	free(buf);
	donereq(r);
	free(a);
	threadexits(nil);
}

// thread #X: receives 9P from v->c, writes machine-independent form to
// the served namespace (v->fd)
void
write9pproc(void *v)
{
	int n;
	char buf[MAXMSG+MAXFDATA];
	Readarg *a = v;
	Req *r;

	while (r = recvp(a->c)) {
		assert(r->magic == Reqmagic);
		if (chatty) {
			threadprint(2, "write %s %F\n", a->name, &r->Fcall);
			// the extra newline breaks up the debugging messages
			// into clusters.
			if (utfrune(a->name, L'→') != nil)
				threadprint(2, "\n");
		}
		if ((n = convS2M(&r->Fcall, buf)) <= 0)
			sysfatal("9p write bad message %d", r->type);
		if (write9p(a->fd, buf, n) != n)
			sysfatal("9p write error: %r");
		donereq(r);
	}
}
