#include <u.h>
#include <libc.h>
#include <auth.h>
#include <fcall.h>
#include <thread.h>
#include "layer.h"
#include "dat.h"
#include "fns.h"

/*
 * internal auth functions make our life (slightly) easier.
 * /sys/src/libauth/authlocal.h
 */
extern	int	_asgetticket(int, char*, char*);
extern	int	_asrdfile(char*, char*, int);
extern	int	_asrdresp(int, char*, int);
extern	int	_asreadn(int, char*, int);

static char *badreq = "bad ticket request";
static char *ccmsg = "can't connect to AS";
static char *srmsg = "server refused authentication";
static char *sgmsg = "server gave up";

// exchange session messages with s->fd, do an authentication dance
int
txsession(Session *s, char *e)
{
	int i;
	Req *rp = newreq(nil, Tsession);

	for (i = 0; i < CHALLEN; i++)
		rp->chal[i] = truerand();
	memmove(s->clientchal, rp->chal, sizeof rp->chal);
	if (sendrecv9p(s, rp, e) < 0) {
		donereq(rp);
		return -1;
	}

	memmove(s->serverchal, rp->chal, sizeof rp->chal);
	strecpy(s->authid, s->authid+NAMELEN, rp->authid);
	strecpy(s->authdom, s->authdom+NAMELEN, rp->authdom);
	s->mcount = 0;
	donereq(rp);

	if (s->authid[0]) {		/* auth required */
		int afd, rv;
		char trbuf[TICKREQLEN];
		Ticketreq tr;

		memset(&tr, 0, sizeof tr);
		tr.type = AuthTreq;
		strcpy(tr.uid, eve);
		_asrdfile("/dev/hostowner", tr.hostid, NAMELEN);
		memmove(tr.authdom, s->authdom, sizeof tr.authdom);
		memmove(tr.authid, s->authid, sizeof tr.authid);
		memmove(tr.chal, s->serverchal, sizeof tr.chal);

		convTR2M(&tr, trbuf);

		/* get ticket */
		afd = authdial();
		if (afd < 0) {
			strcpy(e, ccmsg);
			return -1;
		} else {
			rv = _asgetticket(afd, trbuf, s->tbuf);
			close(afd);
			if (rv < 0) {
				snprint(e, ERRLEN, "getticket: %r");
				return -1;
			}
		}
	} else
		memset(s->tbuf, 0, sizeof s->tbuf);
	return 0;
}

int
setticket(Session *s, Fcall *f, char *e)
{
	int afd, ret = -1;
	char *p;
	char localtbuf[TICKETLEN+4], authbuf[AUTHENTLEN];

	memset(f->ticket, 0, sizeof f->ticket);
	if (strcmp(f->uname, "none") == 0)
		return 0;
	if (strcmp(f->uname, eve) != 0) {
		snprint(e, ERRLEN, "no tickets for %s", f->uname);
		return -1;
	}

	if (s->authid[0] == 0)
		return 0;

	memmove(localtbuf, s->tbuf, TICKETLEN);
	p = localtbuf+TICKETLEN;
	p[0] = s->mcount;
	p[1] = s->mcount>>8;
	p[2] = s->mcount>>16;
	p[3] = s->mcount>>24;
	s->mcount++;

	/* get authenticator */
	afd = open("/dev/authenticator", ORDWR);
	if (afd < 0)
		snprint(e, ERRLEN, "/dev/authenticator: %r");
	else if (write(afd, localtbuf, TICKETLEN+4) < 0)
		snprint(e, ERRLEN, "writing /dev/authenticator: %r");
	else if (read(afd, authbuf, AUTHENTLEN) < 0)
		snprint(e, ERRLEN, "reading /dev/authenticator: %r");
	else {
		memmove(f->ticket, s->tbuf+TICKETLEN, TICKETLEN);
		memmove(f->ticket+TICKETLEN, authbuf, AUTHENTLEN);
		ret = 0;
	}
	if (afd >= 0)
		close(afd);
	return ret;
}

int
checkattach(Session *s, Fcall *f, char *e)
{
	int afd;
	long n;
	char *p;
	char authbuf[TICKETLEN+AUTHENTLEN+CHALLEN+4];

	if (s->authid[0] == 0)
		return 0;

	/* get authenticator from server and check */
	memmove(authbuf, s->tbuf, TICKETLEN);
	memmove(authbuf+TICKETLEN, f->rauth, AUTHENTLEN);
	memmove(authbuf+TICKETLEN+AUTHENTLEN, s->clientchal, CHALLEN);
	p = authbuf+TICKETLEN+AUTHENTLEN+CHALLEN;
	p[0] = (s->mcount-1);
	p[1] = (s->mcount-1)>>8;
	p[2] = (s->mcount-1)>>16;
	p[3] = (s->mcount-1)>>24;

	afd = open("/dev/authcheck", ORDWR);
	if (afd < 0) {
		snprint(e, ERRLEN, "authcheck: %r");
		return -1;
	}

	n = write(afd, authbuf, TICKETLEN+AUTHENTLEN+CHALLEN+4);
	close(afd);
	if (n < 0) {
		memset(authbuf, 0, AUTHENTLEN);
		if (memcmp(authbuf, f->rauth, AUTHENTLEN) == 0)
			snprint(e, ERRLEN, "refused by server");
		else
			snprint(e, ERRLEN, "server lies");
		return -1;
	}
	return 0;
}
